/*
 Copyright (c) 2000-2001, Neil Rhodes and Julie McKeehan
   neil@pobox.com
 All rights reserved.

From the book "Palm Programming, the Developer's Guide, 2nd edition"
by O'Reilly.

Permission granted to use this file however you see fit.
*/

#define DO_NOT_ALLOW_ACCESS_TO_INTERNALS_OF_STRUCTS
#include <BuildDefines.h>
#ifdef DEBUG_BUILD
#define ERROR_CHECK_LEVEL ERROR_CHECK_FULL
#endif
#include <PalmOS.h>
#include "SunAngle_defs.h"
#include "AltForm.h"
#include "MathLib.h"
#include "DateTime.h"
#include "TimeMgr.h"
#include "SunAngle.h"
#include "Utils.h"
#include "pdouble.h"
#include "StringMgr.h"   

RectangleType chartSize;

/******************************************************************************/
static
void AltFormInit(FormPtr form)
{
  // warning-- don't do any drawing in this routine.
  // Also, don't call FrmSetFocus from here (it must be called *after*
  // FrmDrawForm)
}

/******************************************************************************/
static
void AltFormDeinit(FormPtr form)
{
}

/******************************************************************************/
static
void drawLabel (const Char *label, int index) 
{
  double y_inc;
  double x,y;
  int size;
  Char tempStr[10];

  StrPrintF(tempStr, "%s%c", label, 31);
  size = StrLen(tempStr);

  FntSetFont(130);
  y_inc = (chartSize.extent.y)/10.0;  // increment for axis labels
  x = chartSize.topLeft.x-FntCharsWidth(tempStr, size)-2; 
  y = chartSize.topLeft.y+index*y_inc+1;
  WinDrawChars (tempStr, size, x, y-4);
  WinDrawGrayLine (chartSize.topLeft.x, y, chartSize.topLeft.x+chartSize.extent.x-1, y);
}


/******************************************************************************/
static
void drawAxisLabels (sunangleOutputType rise, 
                     sunangleOutputType set,  
                     sunangleOutputType now)
{
  Char sunRiseTime[10];
  Char sunSetTime[10];
//  Char sunNowTime[10];
  Char sunRiseAz[10];
  Char sunSetAz[10];
//  Char sunNowAz[10];
  int size;


  FntSetFont(130);
 
  // Times

  TimeToAscii (rise.sunRise.hours, rise.sunRise.minutes, 
                tfColon, sunRiseTime);
  TimeToAscii (set.sunSet.hours, set.sunSet.minutes, 
                tfColon, sunSetTime);
  size = StrLen(sunRiseTime);
  WinDrawChars (sunRiseTime, size, chartSize.topLeft.x, chartSize.topLeft.y-10);
//  StrCopy (sunSetTime, "12");
  size = StrLen(sunSetTime);
  WinDrawChars (sunSetTime, size, chartSize.topLeft.x+chartSize.extent.x-1-5*(size-1), chartSize.topLeft.y-10);


  // Azimuths

  StrPrintF (sunRiseAz, "%d", (UInt16)(rise.azimuth+0.5));
  StrPrintF (sunSetAz, "%d", (UInt16)(set.azimuth-0.5));
  size = StrLen(sunRiseAz);
  WinDrawChars (sunRiseAz, size, chartSize.topLeft.x, 
                 chartSize.topLeft.y+chartSize.extent.y+2);
  size = StrLen(sunSetAz);
  WinDrawChars (sunSetAz, size, 
                 chartSize.topLeft.x+chartSize.extent.x-3-5*(size-1), 
                 chartSize.topLeft.y+chartSize.extent.y+2);

  // Little nub lines

  WinDrawLine (chartSize.topLeft.x-2, chartSize.topLeft.y-2,
               chartSize.topLeft.x-2, chartSize.topLeft.y-4);
  WinDrawLine (chartSize.topLeft.x+chartSize.extent.x+1, chartSize.topLeft.y-2,
               chartSize.topLeft.x+chartSize.extent.x+1, chartSize.topLeft.y-4);
  WinDrawLine (chartSize.topLeft.x-2, 
               chartSize.topLeft.y+chartSize.extent.y+2,
               chartSize.topLeft.x-2, 
               chartSize.topLeft.y+chartSize.extent.y+4);
  WinDrawLine (chartSize.topLeft.x+chartSize.extent.x+1, 
               chartSize.topLeft.y+chartSize.extent.y+2,
               chartSize.topLeft.x+chartSize.extent.x+1, 
               chartSize.topLeft.y+chartSize.extent.y+4);
}

/******************************************************************************/
static
void  drawChart (FormPtr form)
{
  RectangleType      rect;
  sunangleInputType  sunRiseInput;
  sunangleOutputType sunRiseOutput;
  sunangleInputType  sunSetInput;
  sunangleOutputType sunSetOutput;
  sunangleInputType  sunInput;
  sunangleOutputType sunOutput;
  Char noonTime[10];
  int size;
  
  double minutesInDay;
  double minutesPerPixel;
  
//  Char tempStr[20];

  int i;
  double j,k;

  // chartSize is the actual drawing area.
  // the bounding box is outside it
  chartSize.topLeft.x = 20;
  chartSize.topLeft.y = 30;
  chartSize.extent.x = 158-chartSize.topLeft.x;
  chartSize.extent.y = 158-50;

  // Draw frame, axis labels
  rect.topLeft.x = (chartSize.topLeft.x)-1;
  rect.topLeft.y = (chartSize.topLeft.y)-1;
  rect.extent.x = (chartSize.extent.x)+2;
  rect.extent.y = (chartSize.extent.y)+2;
  WinDrawRectangleFrame (rectangleFrame, &rect);

  drawLabel("100", 0);
  drawLabel("90", 1);
  drawLabel("80", 2);
  drawLabel("70", 3);
  drawLabel("60", 4);
  drawLabel("50", 5);
  drawLabel("40", 6);
  drawLabel("30", 7);
  drawLabel("20", 8);
  drawLabel("10", 9);
  WinDrawChars ("Hor", 3, 
   chartSize.topLeft.x-FntCharsWidth("Hor", 3)-2, 
   chartSize.topLeft.y-5+10*(chartSize.extent.y)/10.0);

  // Calculate sunrise/set for today
  sunInput = *sunangleInput;
  sunRiseInput = *sunangleInput;
  sunSetInput = *sunangleInput;

  CalcSunangle(&sunRiseInput, &sunOutput, true); 

  // check for artic circle in the summer... sun never sets
  // both rise and set times end up equally 12:00
  if ( (sunOutput.sunRise.hours == sunOutput.sunSet.hours) && 
       (sunOutput.sunRise.minutes == sunOutput.sunSet.minutes)) 
  {
    sunOutput.sunRise.hours=0; sunOutput.sunRise.minutes=0;
    sunOutput.sunSet.hours=23; sunOutput.sunSet.minutes=59;
  }
      

  sunRiseInput.dateTime.hour = sunOutput.sunRise.hours;
  sunRiseInput.dateTime.minute = sunOutput.sunRise.minutes;
  sunRiseInput.dateTime.second = 0;

  sunSetInput.dateTime.hour = sunOutput.sunSet.hours;
  sunSetInput.dateTime.minute = sunOutput.sunSet.minutes;
  sunSetInput.dateTime.second = 0;

  CalcSunangle(&sunRiseInput, &sunRiseOutput, true); 
  CalcSunangle(&sunSetInput, &sunSetOutput, true); 

  drawAxisLabels (sunRiseOutput, sunSetOutput, sunOutput);

  // calculate the number of minutes per pixel, and the increment
  j = sunOutput.sunRise.hours*60.0 
       + sunOutput.sunRise.minutes;

  minutesInDay = (sunOutput.sunSet.hours*60.0 
                  + sunOutput.sunSet.minutes)
                 - j;
                 ;
  minutesPerPixel = minutesInDay/chartSize.extent.x;

  k = sunOutput.sunRise.hours*60 + sunOutput.sunRise.minutes;
  k = k + minutesInDay/2;
  
  TimeToAscii (k/60, (Int32)(k+0.5) % 60, tfColon, noonTime);
  size = StrLen(noonTime);
  WinDrawChars (noonTime, size, chartSize.topLeft.x+(chartSize.extent.x/2)-size*5/2, chartSize.topLeft.y-10);

  k = ((sunangleInput->dateTime.hour*60 + sunangleInput->dateTime.minute)-
      (sunOutput.sunRise.hours*60 + sunOutput.sunRise.minutes))
      /minutesPerPixel;

  // draw the altitude
  for (i=0; 
        i<=chartSize.extent.x/2;
        i++) 
  {
    sunInput.dateTime.hour = j/60.0;
    sunInput.dateTime.minute = (int)j % 60; //remainder(j, 60.0, j);
    CalcSunangle(&sunInput, &sunOutput, false);   // "false" speeds calcs
    WinDrawPixel(chartSize.topLeft.x+i, (chartSize.topLeft.y+chartSize.extent.y)-sunOutput.altitude);
    WinDrawPixel(chartSize.topLeft.x+chartSize.extent.x-i, (chartSize.topLeft.y+chartSize.extent.y)-sunOutput.altitude);
    if (i==(int)k )  // draw sun on line
    {
    int lx, ly;
    lx = chartSize.topLeft.x+i;
    ly = (chartSize.topLeft.y+chartSize.extent.y)-sunOutput.altitude;
    WinDrawLine( lx-2, ly-1, lx-2, ly+1);
    WinDrawLine( lx-1, ly-2, lx-1, ly+2);
    WinDrawLine( lx-0, ly-2, lx-0, ly+2);
    WinDrawLine( lx+1, ly-2, lx+1, ly+2);
    WinDrawLine( lx+2, ly-1, lx+2, ly+1);
    }
    if ((chartSize.extent.x-i)==(int)(k) )  // draw sun on line
    {
    int lx, ly;
    lx = chartSize.topLeft.x+chartSize.extent.x-i;
    ly = (chartSize.topLeft.y+chartSize.extent.y)-sunOutput.altitude;
    WinDrawLine( lx-2, ly-1, lx-2, ly+1);
    WinDrawLine( lx-1, ly-2, lx-1, ly+2);
    WinDrawLine( lx-0, ly-2, lx-0, ly+2);
    WinDrawLine( lx+1, ly-2, lx+1, ly+2);
    WinDrawLine( lx+2, ly-1, lx+2, ly+1);
    }
    j += minutesPerPixel;
  }
}

/******************************************************************************/
static Boolean HandleNow()
{
  DateTimeType oldDateTime;
  FormPtr form;
  Boolean update;
  form = FrmGetActiveForm();
  update = false;
 
  if (sunangleInput->now)
  {
    oldDateTime = sunangleInput->dateTime;
    TimSecondsToDateTime (TimGetSeconds(), &sunangleInput->dateTime);
    if (oldDateTime.month != sunangleInput->dateTime.month
        || oldDateTime.day != sunangleInput->dateTime.day
        || oldDateTime.year != sunangleInput->dateTime.year)
    {
        update = true;
    }
    if (oldDateTime.minute != sunangleInput->dateTime.minute
        || oldDateTime.hour != sunangleInput->dateTime.hour)
    {
        update = true;
    }
    if (update)
    {
      FrmEraseForm(form);
      FrmDrawForm(form);
      drawChart (form);
    }
  }
  // we don't return true because we don't pretend to handle the
  // event.  this is because Astro.c doesn't... i think it might
  // be so everyone else sees the nil event.
 
  return false;
}
/******************************************************************************/
Boolean AltFormHandleEvent(EventPtr event)
{
  Boolean handled = false;
  FormPtr form;
//  FieldPtr field;
//  static MemHandle contentHandle;

  form = FrmGetActiveForm();
  switch (event->eType)
  {
  case nilEvent:
    handled = HandleNow();
  break;
  case frmOpenEvent:
    AltFormInit(form);
    WinSetDrawWindow(WinGetDisplayWindow());
    FrmDrawForm(form);
    // here's where you'd add a call to FrmSetFocus
    drawChart(form);
    handled = true;
    break;

  case ctlSelectEvent:
  switch (event->data.ctlSelect.controlID) {
    case b_Alt:
      handled = true;
      break;
    case b_Compass:
      FrmGotoForm(CompassForm);
      handled = true;
      break;
    case b_Listing:
      FrmGotoForm(ListingForm);
      handled = true;
      break;
    case b_Settings:
      FrmGotoForm(SettingsForm);
      handled = true;
      break;
    case b_Info:
      FrmGotoForm(InfoForm);
      handled = true;
      break;
    }
    break;

  case frmCloseEvent:
    AltFormDeinit(FrmGetActiveForm());
    handled = false;
    break;

  default:
    break;
  }
  return handled;
}
